@@ -555,16 +555,12 @@ class XMLFileBacked(FileBacked):
                 xdata = self.xdata.getroottree()
             else:
                 xdata = lxml.etree.parse(fname)
-        included = [el for el in xdata.findall('//' + xinclude)]
-        for el in included:
+        for el in xdata.findall('//' + xinclude):
             name = el.get("href")
             if name.startswith("/"):
                 fpath = name
             else:
-                if fname:
-                    rel = fname
-                else:
-                    rel = self.name
+                rel = fname or self.name
                 fpath = os.path.join(os.path.dirname(rel), name)
 
             # expand globs in xinclude, a bcfg2-specific extension
@@ -579,12 +575,13 @@ class XMLFileBacked(FileBacked):
             parent = el.getparent()
             parent.remove(el)
             for extra in extras:
-                if extra != self.name and extra not in self.extras:
-                    self.extras.append(extra)
+                if extra != self.name:
                     lxml.etree.SubElement(parent, xinclude, href=extra)
-                    self._follow_xincludes(fname=extra)
-                    if extra not in self.extra_monitors:
-                        self.add_monitor(extra)
+                    if extra not in self.extras:
+                        self.extras.append(extra)
+                        self._follow_xincludes(fname=extra)
+                        if extra not in self.extra_monitors:
+                            self.add_monitor(extra)
 
     def Index(self):
         self.xdata = lxml.etree.XML(self.data, base_url=self.name,
@@ -606,15 +603,16 @@ class XMLFileBacked(FileBacked):
 
     def add_monitor(self, fpath):
         """ Add a FAM monitor to a file that has been XIncluded.  This
-        is only done if the constructor got both a ``fam`` object and
-        ``should_monitor`` set to True.
+        is only done if the constructor got a ``fam`` object,
+        regardless of whether ``should_monitor`` is set to True (i.e.,
+        whether or not the base file is monitored).
 
         :param fpath: The full path to the file to monitor
         :type fpath: string
         :returns: None
         """
         self.extra_monitors.append(fpath)
-        if self.fam and self.should_monitor:
+        if self.fam:
             self.fam.AddMonitor(fpath, self)
 
     def __iter__(self):
