@@ -711,12 +711,13 @@ class TestPOSIXTool(TestTool):
             gather_data_rv[idx] = val
         ptool._gather_data.return_value = tuple(gather_data_rv)
 
+        stat_mode = 17407
         mtime = 1344430414
+        stat_rv = (stat_mode, Mock(), Mock(), Mock(), Mock(), Mock(), Mock(),
+                   Mock(), mtime, Mock())
+        gather_data_rv[0] = stat_rv
         entry = reset()
         entry.set("mtime", str(mtime))
-        stat_rv = MagicMock()
-        stat_rv.__getitem__.return_value = mtime
-        gather_data_rv[0] = stat_rv
         ptool._gather_data.return_value = tuple(gather_data_rv)
         self.assertTrue(ptool._verify_metadata(entry))
         ptool._gather_data.assert_called_with(entry.get("name"))
@@ -788,7 +789,7 @@ class TestPOSIXTool(TestTool):
             ptool._gather_data.assert_called_with(entry.get("name"))
             ptool._verify_acls.assert_called_with(entry,
                                                 path=entry.get("name"))
-            mock_matchpathcon.assert_called_with(entry.get("name"), 0)
+            mock_matchpathcon.assert_called_with(entry.get("name"), stat_mode)
             self.assertEqual(entry.get("current_exists", 'true'), 'true')
             for attr, idx, val in expected:
                 self.assertEqual(entry.get(attr), val)
@@ -803,7 +804,7 @@ class TestPOSIXTool(TestTool):
             ptool._gather_data.assert_called_with(entry.get("name"))
             ptool._verify_acls.assert_called_with(entry,
                                                 path=entry.get("name"))
-            mock_matchpathcon.assert_called_with(entry.get("name"), 0)
+            mock_matchpathcon.assert_called_with(entry.get("name"), stat_mode)
             self.assertEqual(entry.get("current_exists", 'true'), 'true')
             for attr, idx, val in expected:
                 self.assertEqual(entry.get(attr), val)
@@ -1009,7 +1010,7 @@ class TestPOSIXTool(TestTool):
             else:
                 return True
         ptool._set_perms.side_effect = set_perms_rv
-        self.assertFalse(ptool._makedirs(entry))
+        self.assertTrue(ptool._makedirs(entry))
         self.assertItemsEqual(mock_exists.call_args_list,
                               [call("/test"), call("/test/foo"),
                                call("/test/foo/bar")])
